from unittest.mock import Mock

from unique_toolkit.content import ContentChunk

from unique_swot.services.citations import CitationManager
from unique_swot.services.report.config import RendererType


def _make_chunk(chunk_id="a", title="Test Doc", start_page=1, end_page=2):
    """Helper to create test ContentChunk."""
    return ContentChunk(
        id="content_1",
        chunk_id=f"chunk_{chunk_id}",
        title=title,
        key="doc.pdf",
        text="Some text",
        start_page=start_page,
        end_page=end_page,
        order=0,
    )


def test_add_citations_docx_format():
    """Test DOCX citation format with page numbers and document reference."""
    registry = Mock()
    registry.retrieve.return_value = _make_chunk()
    manager = CitationManager(content_chunk_registry=registry)

    processed = manager.map_citations_to_report(
        "**References:** [chunk_a]", RendererType.DOCX
    )

    assert "[chunk_a]" not in processed
    assert "p1" in processed or "p2" in processed  # Page numbers included
    assert manager.get_referenced_content_chunks() == [_make_chunk()]


def test_add_citations_chat_format():
    """Test Chat citation format with superscript numbers."""
    registry = Mock()
    registry.retrieve.return_value = _make_chunk()
    manager = CitationManager(content_chunk_registry=registry)

    processed = manager.map_citations_to_report("See ref [chunk_a]", RendererType.CHAT)

    assert "<sup>1</sup>" in processed
    assert "[chunk_a]" not in processed


def test_missing_chunk_uses_placeholder():
    """Test that missing chunks are replaced with [???] placeholder."""
    registry = Mock()
    registry.retrieve.return_value = None
    manager = CitationManager(content_chunk_registry=registry)

    processed = manager.map_citations_to_report(
        "See ref [chunk_missing]", RendererType.DOCX
    )

    assert "[???]" in processed
    assert "[chunk_missing]" not in processed


def test_duplicate_citations_reuse_same_reference():
    """Test that duplicate citations reuse the same formatted reference."""
    registry = Mock()
    registry.retrieve.return_value = _make_chunk()
    manager = CitationManager(content_chunk_registry=registry)

    processed = manager.map_citations_to_report(
        "First [chunk_a] and second [chunk_a]", RendererType.CHAT
    )

    # Both should use the same superscript
    assert processed.count("<sup>1</sup>") == 2
    # Only one chunk should be tracked
    assert len(manager.get_referenced_content_chunks()) == 1


def test_multiple_citations_sequential_numbering():
    """Test that multiple different citations get sequential numbers."""
    registry = Mock()
    registry.retrieve.side_effect = [_make_chunk("a"), _make_chunk("b")]
    manager = CitationManager(content_chunk_registry=registry)

    processed = manager.map_citations_to_report(
        "First [chunk_a] and second [chunk_b]", RendererType.CHAT
    )

    assert "<sup>1</sup>" in processed
    assert "<sup>2</sup>" in processed
    assert len(manager.get_referenced_content_chunks()) == 2


def test_get_references_builds_content_references():
    """Test that get_references creates proper ContentReference objects."""
    registry = Mock()
    registry.retrieve.return_value = _make_chunk()
    manager = CitationManager(content_chunk_registry=registry)
    manager.map_citations_to_report("See [chunk_a]", RendererType.CHAT)

    refs = manager.get_references("msg1")

    assert len(refs) == 1
    assert refs[0].message_id == "msg1"
    assert refs[0].sequence_number == 0
    assert refs[0].source == "SWOT-TOOL"
    assert "unique://content/" in refs[0].url


def test_get_citations_for_docx():
    """Test that get_citations_for_docx returns formatted citation list."""
    registry = Mock()
    registry.retrieve.side_effect = [
        _make_chunk("a", "Doc A"),
        _make_chunk("b", "Doc B"),
    ]
    manager = CitationManager(content_chunk_registry=registry)
    manager.map_citations_to_report("[chunk_a] and [chunk_b]", RendererType.DOCX)

    citations = manager.get_citations_for_docx()

    assert len(citations) == 2
    assert "[1] Doc A" in citations
    assert "[2] Doc B" in citations


def test_page_range_formatting():
    """Test that page ranges are formatted correctly."""
    registry = Mock()
    registry.retrieve.return_value = _make_chunk("a", "Test Doc", 5, 7)
    manager = CitationManager(content_chunk_registry=registry)

    processed = manager.map_citations_to_report("[chunk_a]", RendererType.DOCX)

    # Should include both page numbers
    assert "5" in processed
    assert "7" in processed


def test_single_page_formatting():
    """Test that single page citations are formatted correctly."""
    registry = Mock()
    registry.retrieve.return_value = _make_chunk("a", "Test Doc", 5, 5)
    manager = CitationManager(content_chunk_registry=registry)

    processed = manager.map_citations_to_report("[chunk_a]", RendererType.DOCX)

    assert "p5" in processed



from logging import getLogger

from jinja2 import Template
from unique_toolkit import LanguageModelService
from unique_toolkit._common.validators import LMI
from unique_toolkit.language_model.builder import MessagesBuilder

from unique_swot.services.summarization.config import SummarizationConfig

_LOGGER = getLogger(__name__)


class SummarizationAgent:
    def __init__(
        self,
        *,
        llm: LMI,
        llm_service: LanguageModelService,
        summarization_config: SummarizationConfig,
    ):
        self._llm = llm
        self._llm_service = llm_service
        self.summarization_config = summarization_config

    async def summarize(
        self,
        *,
        company_name: str,
        markdown_report: str,
    ) -> str:
        # Render the report with compatible citations for streaming
        user_prompt = Template(
            self.summarization_config.prompt_config.user_prompt
        ).render(company_name=company_name, report=markdown_report)

        system_prompt = Template(
            self.summarization_config.prompt_config.system_prompt
        ).render()

        messages = (
            MessagesBuilder()
            .system_message_append(system_prompt)
            .user_message_append(user_prompt)
            .build()
        )

        try:
            response = await self._llm_service.complete_async(
                model_name=self._llm.name,
                messages=messages,
            )

            if not isinstance(response.choices[0].message.content, str):
                raise ValueError("Invalid response content received from the LLM")

        except Exception as e:
            # If an error occur during the summarization, we return an empty string
            _LOGGER.exception(f"Error summarizing report: {e}")
            return "An error occurred during generating the executive summary. Please try again."

        summarization_result = response.choices[0].message.content

        return summarization_result



from logging import getLogger
from typing import AsyncIterator, Generator

from jinja2 import Template
from unique_toolkit import LanguageModelService
from unique_toolkit._common.validators import LMI
from unique_toolkit.content import Content

from unique_swot.services.orchestrator.service import StepNotifier
from unique_swot.services.source_management.iteration.config import (
    SourceIterationConfig,
)
from unique_swot.services.source_management.iteration.schema import (
    SourceIterationResults,
)
from unique_swot.utils import (
    generate_structured_output,
    generate_unique_id,
    get_content_chunk_title,
)

_LOGGER = getLogger(__name__)


class SourceIterationAgent:
    def __init__(
        self,
        *,
        llm_service: LanguageModelService,
        llm: LMI,
        source_iteration_config: SourceIterationConfig,
    ):
        self._llm_service = llm_service
        self._llm = llm
        self._source_iteration_config = source_iteration_config
        self._content_map = {}

    async def iterate(
        self, *, contents: list[Content], step_notifier: StepNotifier
    ) -> AsyncIterator[Content]:
        system_prompt = self._compose_system_prompt(
            objective=self._source_iteration_config.prompt_config.objective
        )
        user_prompt = self._compose_user_prompt(contents=contents)

        notification_title = "Sorting sources"

        await step_notifier.notify(
            title=notification_title,
        )

        response = await generate_structured_output(
            system_prompt=system_prompt,
            user_message=user_prompt,
            llm=self._llm,
            llm_service=self._llm_service,
            output_model=SourceIterationResults,
        )

        fallback_notification_message = (
            "Unable to sort the sources. Original order will be preserved."
        )

        notification_description = (
            response.results_summary if response else fallback_notification_message
        )

        await step_notifier.notify(
            title=notification_title,
            description=notification_description,
            completed=True,
            progress=100,
        )

        # Return the ordered (and missed) contents as an AsyncIterator
        return self._handle_results(results=response)

    def _compose_system_prompt(self, *, objective: str) -> str:
        template = Template(self._source_iteration_config.prompt_config.system_prompt)
        return template.render(objective=objective)

    def _compose_user_prompt(self, *, contents: list[Content]) -> str:
        contents_list = list(self._return_contents(contents=contents))

        template = Template(self._source_iteration_config.prompt_config.user_prompt)
        return template.render(contents=contents_list)

    def _return_contents(
        self, *, contents: list[Content]
    ) -> Generator[dict[str, str], None, None]:
        for content in contents:
            unique_id = generate_unique_id(prefix="content")
            self._content_map[unique_id] = content
            selected_chunks = content.chunks[
                : self._source_iteration_config.max_number_of_selected_chunks
            ]
            selected_chunks_texts = [chunk.text for chunk in selected_chunks]
            yield {
                "id": unique_id,
                "document_title": get_content_chunk_title(content),
                "chunks": "\n".join(selected_chunks_texts),
            }

    def _handle_results(
        self, *, results: SourceIterationResults | None
    ) -> AsyncIterator[Content]:
        if results is None:
            # If no results, return all contents in original order
            async def _default_generator():
                for content in self._content_map.values():
                    yield content

            return _default_generator()

        async def _generator():
            # Create a mapping of ID to order for efficient lookup
            id_to_order = {
                result.id: result.order for result in results.ordered_sources
            }

            # Separate missed and ordered contents in one pass
            missed_contents = []
            ordered_items = []

            for content_id, content in self._content_map.items():
                if content_id in id_to_order:
                    ordered_items.append((id_to_order[content_id], content))
                else:
                    missed_contents.append(content)

            # Yield missed documents first
            for content in missed_contents:
                yield content

            # Sort and yield ordered documents
            ordered_items.sort(key=lambda x: x[0])  # Sort by order
            for _, content in ordered_items:
                yield content

        return _generator()



"""Tests for the ReportDeliveryService."""

from unittest.mock import Mock, patch

import pytest

from unique_swot.services.generation.models.base import (
    SWOTReportComponents,
    SWOTReportComponentSection,
    SWOTReportSectionEntry,
)
from unique_swot.services.report.config import RendererType
from unique_swot.services.report.delivery import ReportDeliveryService
from unique_swot.services.session.schema import (
    SwotAnalysisSessionConfig,
    UniqueCompanyListing,
)


def _make_report_components():
    """Helper to create test SWOTReportComponents."""
    return SWOTReportComponents(
        strengths=[
            SWOTReportComponentSection(
                h2="Market Position",
                entries=[
                    SWOTReportSectionEntry(
                        preview="Strong brand",
                        content="Strong brand recognition [chunk_a]",
                    )
                ],
            )
        ],
        weaknesses=[],
        opportunities=[],
        threats=[],
    )


def _make_session_config():
    """Helper to create test session config."""
    return SwotAnalysisSessionConfig(
        company_listing=UniqueCompanyListing(
            sourceRef=123.0,
            name="ACME Corp",
            display_name="ACME Corporation",
            country="US",
            tickers=[],
            source_url="https://example.com/acme",
            source="test",
        ),
        use_earnings_call=False,
        use_web_sources=False,
        earnings_call_start_date=None,
    )


def test_render_report_applies_citations(mock_citation_manager):
    """Test that render_report applies citations to the report."""
    mock_citation_manager.map_citations_to_report.return_value = "Report with citations"

    chat_service = Mock()
    docx_renderer = Mock()

    renderer_config = Mock()
    renderer_config.report_structure_template = "Template: {{executive_summary}}"
    renderer_config.renderer_type = RendererType.CHAT

    service = ReportDeliveryService(
        chat_service=chat_service,
        docx_renderer=docx_renderer,
        citation_manager=mock_citation_manager,
        renderer_config=renderer_config,
    )

    result = service.render_report(
        citation_fn=lambda x: x + " [cited]",
        executive_summary="Summary",
    )

    assert "[cited]" in result


def test_render_report_uses_template(mock_citation_manager):
    """Test that render_report uses the Jinja template."""
    mock_citation_manager.map_citations_to_report.return_value = "Processed"

    chat_service = Mock()
    docx_renderer = Mock()

    renderer_config = Mock()
    renderer_config.report_structure_template = (
        "Executive Summary: {{executive_summary}}"
    )
    renderer_config.report_body_template = (
        "Strengths: {% for s in strengths %}{{ s.h2 }}{% endfor %}"
    )
    renderer_config.renderer_type = RendererType.CHAT

    service = ReportDeliveryService(
        chat_service=chat_service,
        docx_renderer=docx_renderer,
        citation_manager=mock_citation_manager,
        renderer_config=renderer_config,
    )

    body = service.render_body(_make_report_components())

    assert "Market Position" in body


def test_deliver_report_docx_mode(mock_citation_manager):
    """Test delivering report in DOCX mode."""
    mock_citation_manager.map_citations_to_report.return_value = "Report [1]"
    mock_citation_manager.get_citations_for_docx.return_value = ["[1] Source"]

    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.upload_to_chat_from_bytes.return_value = Mock(
        id="uploaded_id", title="Report", key="report.docx"
    )

    docx_renderer = Mock()

    renderer_config = Mock()
    renderer_config.report_structure_template = "{{executive_summary}}\n{{body}}"
    renderer_config.renderer_type = RendererType.DOCX

    service = ReportDeliveryService(
        chat_service=chat_service,
        docx_renderer=docx_renderer,
        citation_manager=mock_citation_manager,
        renderer_config=renderer_config,
    )

    with patch(
        "unique_swot.services.report.delivery.convert_markdown_to_docx",
        return_value=b"docx bytes",
    ):
        service.deliver_report(
            executive_summary="Summary",
            body="Body content",
            session_config=_make_session_config(),
            docx_template_fields={"title": "SWOT Report"},
            ingest_docx=True,
        )

        # Verify DOCX was uploaded
        chat_service.upload_to_chat_from_bytes.assert_called_once()
        # Verify message was modified
        chat_service.modify_assistant_message.assert_called_once()


def test_deliver_report_chat_mode(mock_citation_manager):
    """Test delivering report in CHAT mode."""
    mock_citation_manager.map_citations_to_report.return_value = "Report with citations"
    mock_citation_manager.get_references.return_value = []
    mock_citation_manager.get_citations_for_docx.return_value = []

    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"

    docx_renderer = Mock()

    renderer_config = Mock()
    renderer_config.report_structure_template = "{{executive_summary}}\n{{body}}"
    renderer_config.renderer_type = RendererType.CHAT

    service = ReportDeliveryService(
        chat_service=chat_service,
        docx_renderer=docx_renderer,
        citation_manager=mock_citation_manager,
        renderer_config=renderer_config,
    )

    service.deliver_report(
        executive_summary="Summary",
        body="Body content",
        session_config=_make_session_config(),
        docx_template_fields={"title": "SWOT Report"},
        ingest_docx=False,
    )

    # Verify message was modified with markdown content
    chat_service.modify_assistant_message.assert_called_once()
    call_kwargs = chat_service.modify_assistant_message.call_args.kwargs
    assert "Report with citations" in call_kwargs["content"]


def test_deliver_report_docx_with_citations_footer(mock_citation_manager):
    """Test that DOCX report includes citation footer."""
    mock_citation_manager.map_citations_to_report.return_value = "Report"
    mock_citation_manager.get_citations_for_docx.return_value = [
        "[1] Document A",
        "[2] Document B",
    ]

    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.upload_to_chat_from_bytes.return_value = Mock(
        id="uploaded_id", title="Report", key="report.docx"
    )

    docx_renderer = Mock()

    renderer_config = Mock()
    renderer_config.report_structure_template = "{{executive_summary}}\n{{body}}\n{% if citations %}Citations: {{citations}}{% endif %}"
    renderer_config.renderer_type = RendererType.DOCX

    service = ReportDeliveryService(
        chat_service=chat_service,
        docx_renderer=docx_renderer,
        citation_manager=mock_citation_manager,
        renderer_config=renderer_config,
    )

    with patch(
        "unique_swot.services.report.delivery.convert_markdown_to_docx",
        return_value=b"docx bytes",
    ) as mock_convert:
        service.deliver_report(
            executive_summary="Summary",
            body="Body content",
            session_config=_make_session_config(),
            docx_template_fields={"title": "SWOT Report"},
            ingest_docx=True,
        )

        # Verify conversion was called
        mock_convert.assert_called_once()


def test_deliver_report_skip_docx_ingestion(mock_citation_manager):
    """Test that DOCX ingestion can be skipped."""
    mock_citation_manager.map_citations_to_report.return_value = "Report"
    mock_citation_manager.get_citations_for_docx.return_value = []

    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.upload_to_chat_from_bytes.return_value = Mock(
        id="uploaded_id", title="Report", key="report.docx"
    )

    docx_renderer = Mock()

    renderer_config = Mock()
    renderer_config.report_structure_template = "{{executive_summary}}\n{{body}}"
    renderer_config.renderer_type = RendererType.DOCX

    service = ReportDeliveryService(
        chat_service=chat_service,
        docx_renderer=docx_renderer,
        citation_manager=mock_citation_manager,
        renderer_config=renderer_config,
    )

    with patch(
        "unique_swot.services.report.delivery.convert_markdown_to_docx",
        return_value=b"docx bytes",
    ):
        service.deliver_report(
            executive_summary="Summary",
            body="Body content",
            session_config=_make_session_config(),
            docx_template_fields={"title": "SWOT Report"},
            ingest_docx=False,  # Skip ingestion
        )

        # Verify upload was called with skip_ingestion=True
        call_kwargs = chat_service.upload_to_chat_from_bytes.call_args.kwargs
        assert call_kwargs["skip_ingestion"] is True


def test_deliver_report_with_existing_references(mock_citation_manager):
    """Test delivering report with existing references."""
    mock_citation_manager.map_citations_to_report.return_value = "Report"
    mock_citation_manager.get_citations_for_docx.return_value = [
        "[1] Doc A",
        "[2] Doc B",
    ]

    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.upload_to_chat_from_bytes.return_value = Mock(
        id="uploaded_id", title="Report", key="report.docx"
    )

    docx_renderer = Mock()

    renderer_config = Mock()
    renderer_config.report_structure_template = "{{executive_summary}}\n{{body}}"
    renderer_config.renderer_type = RendererType.DOCX

    service = ReportDeliveryService(
        chat_service=chat_service,
        docx_renderer=docx_renderer,
        citation_manager=mock_citation_manager,
        renderer_config=renderer_config,
    )

    with patch(
        "unique_swot.services.report.delivery.convert_markdown_to_docx",
        return_value=b"docx bytes",
    ):
        service.deliver_report(
            executive_summary="Summary",
            body="Body content",
            session_config=_make_session_config(),
            docx_template_fields={"title": "SWOT Report"},
            ingest_docx=True,
        )

        # Verify reference number is used in message
        call_kwargs = chat_service.modify_assistant_message.call_args.kwargs
        assert "<sup>2</sup>" in call_kwargs["content"]


def test_deliver_report_invalid_renderer_type_raises(mock_citation_manager):
    """Test that invalid renderer type raises ValueError."""
    mock_citation_manager.map_citations_to_report.return_value = "Report"
    mock_citation_manager.get_citations_for_docx.return_value = []

    chat_service = Mock()
    docx_renderer = Mock()

    renderer_config = Mock()
    renderer_config.report_structure_template = "{{executive_summary}}\n{{body}}"
    renderer_config.renderer_type = RendererType.DOCX

    # Create service with valid enum, then mock the enum value to trigger error
    service = ReportDeliveryService(
        chat_service=chat_service,
        docx_renderer=docx_renderer,
        citation_manager=mock_citation_manager,
        renderer_config=renderer_config,
    )

    # Simulate invalid renderer type by patching the internal value
    service._renderer_config.renderer_type = "invalid"  # type: ignore

    with pytest.raises(ValueError):
        service.deliver_report(
            executive_summary="Summary",
            body="Body content",
            session_config=_make_session_config(),
            docx_template_fields={"title": "SWOT Report"},
            ingest_docx=True,
        )



from unique_swot.services.source_management.collection.sources.earnings_call import (
    collect_earnings_calls,
)
from unique_swot.services.source_management.collection.sources.knowledge_base import (
    collect_knowledge_base,
)
from unique_swot.services.source_management.collection.sources.web import (
    collect_web_sources,
)

__all__ = [
    "collect_earnings_calls",
    "collect_knowledge_base",
    "collect_web_sources",
]



from pydantic import Field

from unique_swot.utils import StructuredOutputResult


class SourceIterationResult(StructuredOutputResult):
    """This class is responsible for the result of the source iteration."""

    id: str = Field(description="The id of the source")
    order: int = Field(description="The order of the source")


class SourceIterationResults(StructuredOutputResult):
    """This class is responsible for the results of the source iteration."""

    results_summary: str = Field(
        description="A detailed description of the ordering outcome."
    )
    ordered_sources: list[SourceIterationResult] = Field(
        description="The list of sources ordered from the oldest to the newest"
    )



from pydantic import BaseModel, Field
from unique_toolkit._common.pydantic_helpers import get_configuration_dict
from unique_toolkit._common.validators import LMI, get_LMI_default_field
from unique_toolkit.language_model.infos import LanguageModelName

from unique_swot.services.summarization.prompts.config import SummarizationPromptConfig


class SummarizationConfig(BaseModel):
    model_config = get_configuration_dict()

    language_model: LMI = get_LMI_default_field(
        LanguageModelName.AZURE_GPT_4o_2024_1120,
        description="The language model to use for the summarization.",
    )

    prompt_config: SummarizationPromptConfig = Field(
        default_factory=SummarizationPromptConfig,
        description="The configuration for the summarization prompts.",
    )



"""Tests for the StepNotifier service."""

from unittest.mock import AsyncMock, Mock

import pytest
from unique_toolkit.content import ContentReference

from unique_swot.services.notification.notifier import StepNotifier


@pytest.mark.asyncio
async def test_step_notifier_basic_notification():
    """Test basic notification without sources or progress."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    await notifier.notify(title="Test Title", description="Test Description")

    # Verify notification was logged (implementation detail may vary)
    assert notifier._log_registry is not None


@pytest.mark.asyncio
async def test_step_notifier_with_progress():
    """Test notification with progress percentage."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    await notifier.notify(title="Processing", description="Step 1 of 3", progress=33)

    # Verify progress was tracked
    assert notifier._log_registry is not None


@pytest.mark.asyncio
async def test_step_notifier_with_sources():
    """Test notification with content references."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    sources = [
        ContentReference(
            url="unique://content/123",
            source_id="source_1",
            message_id="msg_123",
            name="Test Document",
            sequence_number=0,
            source="SWOT-TOOL",
        )
    ]

    await notifier.notify(title="Processing Document", sources=sources)

    # Verify sources were included
    assert notifier._log_registry is not None


@pytest.mark.asyncio
async def test_step_notifier_completed_flag():
    """Test notification with completed flag."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    await notifier.notify(
        title="Analysis Complete",
        description="All steps finished",
        progress=100,
        completed=True,
    )

    # Verify completion was tracked
    assert notifier._log_registry is not None


@pytest.mark.asyncio
async def test_step_notifier_multiple_notifications():
    """Test multiple sequential notifications."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    await notifier.notify(title="Step 1", progress=0)
    await notifier.notify(title="Step 2", progress=50)
    await notifier.notify(title="Step 3", progress=100, completed=True)

    # All notifications should be tracked
    assert notifier._log_registry is not None


@pytest.mark.asyncio
async def test_step_notifier_get_total_references():
    """Test getting total number of references."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    # Initial count should be 0
    assert notifier.get_total_number_of_references() == 0


@pytest.mark.asyncio
async def test_step_notifier_with_all_parameters():
    """Test notification with all parameters provided."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    sources = [
        ContentReference(
            url="unique://content/123",
            source_id="source_1",
            message_id="msg_123",
            name="Document 1",
            sequence_number=0,
            source="SWOT-TOOL",
        ),
        ContentReference(
            url="unique://content/456",
            source_id="source_2",
            message_id="msg_123",
            name="Document 2",
            sequence_number=1,
            source="SWOT-TOOL",
        ),
    ]

    await notifier.notify(
        title="Processing Complete",
        description="Analyzed 2 documents",
        sources=sources,
        progress=100,
        completed=True,
    )

    # Verify all parameters were handled
    assert notifier._log_registry is not None


@pytest.mark.asyncio
async def test_step_notifier_empty_description():
    """Test notification with empty description (default)."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    await notifier.notify(title="Title Only")

    # Should work with default empty description
    assert notifier._log_registry is not None


@pytest.mark.asyncio
async def test_step_notifier_empty_sources():
    """Test notification with empty sources list (default)."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    await notifier.notify(title="No Sources", sources=[])

    # Should work with empty sources
    assert notifier._log_registry is not None


@pytest.mark.asyncio
async def test_step_notifier_none_progress():
    """Test notification with None progress (default)."""
    chat_service = Mock()
    chat_service.assistant_message_id = "msg_123"
    chat_service.create_message_log_async = AsyncMock(
        return_value=Mock(message_log_id="log_123")
    )
    chat_service.update_message_log_async = AsyncMock()

    notifier = StepNotifier(chat_service=chat_service)

    await notifier.notify(title="Indeterminate Progress", progress=None)

    # Should work with None progress
    assert notifier._log_registry is not None



from logging import getLogger
from typing import AsyncIterator, Generator, Protocol

from tqdm.asyncio import tqdm
from unique_toolkit.chat.service import ChatService
from unique_toolkit.content import Content, ContentChunk, ContentReference

from unique_swot.services.generation.models.base import SWOTReportComponents
from unique_swot.services.memory.base import SwotMemoryService
from unique_swot.services.schemas import SWOTPlan
from unique_swot.services.source_management.selection.schema import (
    SourceSelectionResult,
)

_LOGGER = getLogger(__name__)


class StepNotifier(Protocol):
    """This class is responsible for notifying the user of the progress of a specific step of the SWOT analysis."""

    async def notify(
        self,
        title: str,
        description: str = "",
        sources: list[ContentReference] = [],
        progress: int | None = None,
        completed: bool = False,
    ): ...


class ProgressNotifier(Protocol):
    """This class is responsible for notifying the user of the progress of the SWOT analysis."""

    async def update(self, *, progress: int | float, title: str | None = None): ...

    async def increment(self, fraction: float): ...

    @property
    def step_size(self) -> int | float: ...

    @step_size.setter
    def step_size(self, value: int | float) -> None: ...


class SourceCollector(Protocol):
    """This class is responsible for collecting the sources from the various data sources."""

    async def collect(self, *, step_notifier: StepNotifier) -> list[Content]: ...


class SourceSelector(Protocol):
    """This class is responsible for selecting the sources that are most relevant to the SWOT analysis."""

    async def select(
        self, *, company_name: str, content: Content, step_notifier: StepNotifier
    ) -> SourceSelectionResult: ...


class SourceIterator(Protocol):
    """This class is responsible for prioritizing the sources that are most relevant to the SWOT analysis."""

    async def iterate(
        self, *, contents: list[Content], step_notifier: StepNotifier
    ) -> AsyncIterator[Content]: ...


class SourceRegistry(Protocol):
    """This class is responsible for registering the sources that are processed."""

    def register(self, *, chunk: ContentChunk) -> str: ...

    def retrieve(self, *, id: str) -> ContentChunk | None: ...


class ReportingAgent(Protocol):
    """This class is responsible for generating the SWOT report."""

    async def generate(
        self,
        *,
        plan: SWOTPlan,
        content: Content,
        source_registry: SourceRegistry,
        step_notifier: StepNotifier,
        progress_notifier: ProgressNotifier,
    ) -> None: ...

    def get_reports(self) -> SWOTReportComponents: ...


class SWOTOrchestrator:
    def __init__(
        self,
        step_notifier: StepNotifier,
        source_collector: SourceCollector,
        source_selector: SourceSelector,
        source_iterator: SourceIterator,
        reporting_agent: ReportingAgent,
        source_registry: SourceRegistry,
        progress_notifier: ProgressNotifier,
        memory_service: SwotMemoryService,
        chat_service: ChatService,
    ):
        self._source_collector = source_collector
        self._source_selector = source_selector
        self._reporting_agent = reporting_agent
        self._source_iterator = source_iterator
        self._memory_service = memory_service
        self._step_notifier = step_notifier
        self._source_registry = source_registry
        self._progress_notifier = progress_notifier
        self._chat_service = chat_service

    async def run(self, *, company_name: str, plan: SWOTPlan) -> SWOTReportComponents:
        contents = await self._source_collector.collect(
            step_notifier=self._step_notifier
        )

        await self._progress_notifier.update(progress=5)

        source_iterator = await self._source_iterator.iterate(
            contents=contents, step_notifier=self._step_notifier
        )

        await self._progress_notifier.update(progress=10)

        total_steps = len(contents)

        if total_steps == 0:
            # Early return if there are no sources to process (this should never happen)
            raise ValueError("No sources to process")

        step_size, progress_sequence = _create_progress_sequence(
            start=10, stop=80, steps=total_steps
        )

        # This is for "agent generating" progress tracking
        self._progress_notifier.step_size = step_size

        async for content in tqdm(
            source_iterator, total=total_steps, desc="Processing sources"
        ):
            await self._progress_notifier.update(
                progress=next(progress_sequence),
            )

            source_selection_result = await self._source_selector.select(
                company_name=company_name,
                content=content,
                step_notifier=self._step_notifier,
            )

            if not source_selection_result.should_select:
                # Skip the source if it is not selected
                _LOGGER.info("Skipping source because it is not selected")
                continue

            _LOGGER.info("Selecting source because it is selected")

            await self._reporting_agent.generate(
                plan=plan,
                content=content,
                step_notifier=self._step_notifier,
                source_registry=self._source_registry,
                progress_notifier=self._progress_notifier,
            )

        return self._reporting_agent.get_reports()


def _create_progress_sequence(
    start: int, stop: int, steps: int
) -> tuple[float, Generator[float, None, None]]:
    step_size = (stop - start) / steps

    def frange(start):
        if steps == 0:
            raise ValueError("step must not be zero")

        for i in range(steps):
            yield start + i * step_size

    return step_size, frange(start)



from datetime import datetime
from logging import getLogger

from pydantic import BaseModel, ConfigDict
from unique_quartr.service import QuartrService
from unique_toolkit import KnowledgeBaseService
from unique_toolkit._common.docx_generator import DocxGeneratorService
from unique_toolkit.content import Content

from unique_swot.services.orchestrator.service import StepNotifier
from unique_swot.services.session.schema import UniqueCompanyListing
from unique_swot.services.source_management.collection.sources import (
    collect_earnings_calls,
    collect_knowledge_base,
    collect_web_sources,
)

_LOGGER = getLogger(__name__)


class CollectionContext(BaseModel):
    model_config = ConfigDict(frozen=True)

    company: UniqueCompanyListing
    use_earnings_calls: bool
    upload_scope_id_earnings_calls: str
    earnings_call_start_date: datetime
    use_web_sources: bool
    metadata_filter: dict | None


class SourceCollectionManager:
    def __init__(
        self,
        *,
        context: CollectionContext,
        knowledge_base_service: KnowledgeBaseService,
        quartr_service: QuartrService | None = None,
        earnings_call_docx_generator_service: DocxGeneratorService,
    ):
        self._context = context
        self._knowledge_base_service = knowledge_base_service
        self._quartr_service = quartr_service
        self._earnings_call_docx_generator_service = (
            earnings_call_docx_generator_service
        )

    @property
    def notification_title(self) -> str:
        return "**Collecting Sources**"

    async def collect(self, *, step_notifier: StepNotifier) -> list[Content]:
        await step_notifier.notify(title=self.notification_title, progress=0)

        internal_documents_sources = await self._collect_internal_documents(
            metadata_filter=self._context.metadata_filter, step_notifier=step_notifier
        )

        earnings_calls_sources = await self._collect_earnings_calls(
            quartr_service=self._quartr_service,
            use_earnings_calls=self._context.use_earnings_calls,
            upload_scope_id_earnings_calls=self._context.upload_scope_id_earnings_calls,
            company=self._context.company,
            docx_generator_service=self._earnings_call_docx_generator_service,
            knowledge_base_service=self._knowledge_base_service,
            earnings_call_start_date=self._context.earnings_call_start_date,
            step_notifier=step_notifier,
        )

        web_sources = await self._collect_web_sources(
            use_web_sources=self._context.use_web_sources,
            step_notifier=step_notifier,
        )

        all_sources = internal_documents_sources + earnings_calls_sources + web_sources

        await step_notifier.notify(
            title=self.notification_title,
            description=f"Collected {len(all_sources)} sources!",
            progress=100,
            completed=True,
        )

        return all_sources

    async def _collect_earnings_calls(
        self,
        *,
        use_earnings_calls: bool,
        quartr_service: QuartrService | None,
        upload_scope_id_earnings_calls: str | None,
        company: UniqueCompanyListing,
        earnings_call_start_date: datetime,
        docx_generator_service: DocxGeneratorService,
        knowledge_base_service: KnowledgeBaseService,
        step_notifier: StepNotifier,
    ) -> list[Content]:
        if not use_earnings_calls:
            _LOGGER.warning("No earnings calls will be collected.")
            return []

        if quartr_service is None:
            _LOGGER.error(
                "Quartr service is not provided. Check that your company has access to Quartr API."
            )
            await step_notifier.notify(
                title=self.notification_title,
                description="Cannot collect earnings calls. Quartr service is not provided. Check that your company has access to Quartr API.",
                progress=0,
                completed=True,
            )
            return []

        if not upload_scope_id_earnings_calls:
            _LOGGER.error(
                "Upload scope ID for earnings calls is not provided. This is a mandatory parameter when using earnings calls as a data source."
            )
            await step_notifier.notify(
                title=self.notification_title,
                description="Cannot collect earnings calls. Upload scope ID for earnings calls is not provided. This is a mandatory parameter when using earnings calls as a data source.",
                progress=0,
                completed=True,
            )
            return []

        _LOGGER.info("Collecting earnings calls!")

        contents = await collect_earnings_calls(
            quartr_service=quartr_service,
            upload_scope_id=upload_scope_id_earnings_calls,
            company=company,
            earnings_call_start_date=earnings_call_start_date,
            docx_generator_service=docx_generator_service,
            knowledge_base_service=knowledge_base_service,
        )

        await step_notifier.notify(
            title=self.notification_title,
            description=f"Collected {len(contents)} earnings call(s)!",
            progress=0,
        )
        return contents

    async def _collect_web_sources(
        self,
        *,
        use_web_sources: bool,
        step_notifier: StepNotifier,
    ) -> list[Content]:
        if not use_web_sources:
            _LOGGER.warning("No web sources will be collected.")
            return []
        _LOGGER.info("Collecting web sources!")

        return collect_web_sources()

    async def _collect_internal_documents(
        self,
        *,
        metadata_filter: dict | None,
        step_notifier: StepNotifier,
    ) -> list[Content]:
        if metadata_filter is None:
            _LOGGER.warning(
                "No where clause provided. No internal documents will be collected."
            )
            return []

        _LOGGER.info("Collecting internal documents!")

        contents = await collect_knowledge_base(
            knowledge_base_service=self._knowledge_base_service,
            metadata_filter=metadata_filter,
        )
        await step_notifier.notify(
            title=self.notification_title,
            description=f"Collected {len(contents)} internal documents!",
            progress=0,
        )
        return contents



import asyncio
from enum import StrEnum
from typing import Any, Awaitable, Callable, Sequence


class ExecutionMode(StrEnum):
    SEQUENTIAL = "Sequential"
    CONCURRENT = "Concurrent"


class AgenticPlanExecutor:
    """
    Register async callables with their arguments and run them sequentially or
    concurrently. Execution captures exceptions instead of raising them so
    callers can inspect failures.
    """

    def __init__(
        self, *, execution_mode: ExecutionMode, max_concurrent_tasks: int | None = None
    ):
        self._execution_mode = execution_mode
        self._max_concurrent_tasks = max_concurrent_tasks or 10
        self._queue: list[
            tuple[Callable[..., Awaitable[Any]], tuple[Any, ...], dict[str, Any]]
        ] = []

    def add(self, fn: Callable[..., Awaitable[Any]], /, *args: Any, **kwargs: Any):
        """Register an awaitable function with its arguments."""
        self._queue.append((fn, args, kwargs))

    async def run(self) -> Sequence[Exception | Any]:
        """Execute all registered tasks and return per-task exceptions (or None)."""
        try:
            match self._execution_mode:
                case ExecutionMode.SEQUENTIAL:
                    return await self._run_sequential()
                case ExecutionMode.CONCURRENT:
                    return await self._run_concurrent()
                case _:
                    raise ValueError(f"Invalid execution mode: {self._execution_mode}")
        finally:
            # Always clear the queue after running to avoid memory leaks
            # and to avoid running the same tasks multiple times
            self._queue.clear()

    async def _run_sequential(self) -> Sequence[Exception | Any]:
        results: list[Exception | None] = []
        for fn, args, kwargs in self._queue:
            try:
                result = await fn(*args, **kwargs)
                results.append(result)
            except Exception as exc:  # noqa: BLE001
                results.append(exc)
        return results

    async def _run_concurrent(self) -> Sequence[Exception | Any]:
        semaphore = asyncio.Semaphore(self._max_concurrent_tasks)

        async def _runner(
            fn: Callable[..., Awaitable[Any]],
            args: tuple[Any, ...],
            kwargs: dict[str, Any],
        ):
            async with semaphore:
                try:
                    result = await fn(*args, **kwargs)
                    return result
                except Exception as exc:  # noqa: BLE001
                    return exc

        tasks = [
            asyncio.create_task(_runner(fn, args, kwargs))
            for fn, args, kwargs in self._queue
        ]
        return await asyncio.gather(*tasks)



from types import SimpleNamespace
from unittest.mock import AsyncMock, Mock

import pytest
from pydantic import BaseModel

from unique_swot.utils import generate_structured_output


class DummyOutput(BaseModel):
    value: str


def _build_llm():
    # Minimal LMI-like object with the encoder name used by tiktoken
    return SimpleNamespace(name="dummy-model", encoder_name="cl100k_base")


@pytest.mark.asyncio
async def test_generate_structured_output_retries_and_succeeds(monkeypatch):
    """Should retry failed calls and return the parsed structured model on success."""

    async def _raise_then_succeed(*_, **__):
        # First two attempts fail, third succeeds
        if call_counter["count"] < 2:
            call_counter["count"] += 1
            raise RuntimeError("temporary failure")
        message = Mock(parsed={"value": "ok"})
        return Mock(choices=[Mock(message=message)])

    call_counter = {"count": 0}
    llm_service = Mock()
    llm_service.complete_async = AsyncMock(side_effect=_raise_then_succeed)

    result = await generate_structured_output(
        user_message="hi",
        system_prompt="sys",
        llm=_build_llm(),
        output_model=DummyOutput,
        llm_service=llm_service,
    )

    assert isinstance(result, DummyOutput)
    assert result.value == "ok"
    assert llm_service.complete_async.await_count == 3


@pytest.mark.asyncio
async def test_generate_structured_output_returns_none_after_max_retries():
    """Should return None after exhausting retries."""

    llm_service = Mock()
    llm_service.complete_async = AsyncMock(side_effect=RuntimeError("fail"))

    result = await generate_structured_output(
        user_message="hi",
        system_prompt="sys",
        llm=_build_llm(),
        output_model=DummyOutput,
        llm_service=llm_service,
    )

    assert result is None
    assert llm_service.complete_async.await_count == 3



from logging import getLogger

from unique_toolkit._common.docx_generator import (
    DocxGeneratorService,
)

_LOGGER = getLogger(__name__)


def convert_markdown_to_docx(
    markdown_content: str,
    docx_generator_service: DocxGeneratorService,
    fields: dict[str, str],
) -> bytes | None:
    docx_elements = docx_generator_service.parse_markdown_to_list_content_fields(
        markdown_content
    )
    return docx_generator_service.generate_from_template(docx_elements, fields)



from pydantic import BaseModel, Field
from unique_toolkit._common.pydantic_helpers import get_configuration_dict

from unique_swot.services.generation.agentic.prompts.commands.create_new_section.config import (
    CreateNewSectionPromptConfig,
)
from unique_swot.services.generation.agentic.prompts.commands.update_existing_section.config import (
    UpdateExistingSectionPromptConfig,
)


class CommandsPromptConfig(BaseModel):
    model_config = get_configuration_dict()

    create_new_section_prompt_config: CreateNewSectionPromptConfig = Field(
        default=CreateNewSectionPromptConfig(),
        description="The prompt config for the create new section command.",
    )
    update_existing_section_prompt_config: UpdateExistingSectionPromptConfig = Field(
        default=UpdateExistingSectionPromptConfig(),
        description="The prompt config for the update existing section command.",
    )



from datetime import datetime
from enum import StrEnum
from pathlib import Path

from jinja2 import Template
from pydantic import BaseModel, Field
from unique_quartr.endpoints.schemas import TickerDto
from unique_toolkit._common.pydantic_helpers import get_configuration_dict

from unique_swot.utils import load_template

# Get the directory containing this file
_PROMPTS_DIR = Path(__file__).parent
SESSION_INFO_TEMPLATE: str = load_template(
    _PROMPTS_DIR, template_name="session_info.j2"
)


class SessionState(StrEnum):
    RUNNING = "Running"
    COMPLETED = "Completed"
    FAILED = "Failed"


class UniqueCompanyListing(BaseModel):
    model_config = get_configuration_dict()
    id: float = Field(
        ...,
        description="The id of the company",
        validation_alias="sourceRef",
        alias="sourceRef",
    )
    name: str = Field(..., description="The name of the company")
    display_name: str = Field(
        ...,
        description="The display name of the company",
    )
    country: str = Field(..., description="The country of the company")
    tickers: list[TickerDto] = Field(..., description="The tickers of the company")
    source_url: str = Field(..., description="The source URL of the company")
    source: str = Field(..., description="The source of the company")


class SwotAnalysisSessionConfig(BaseModel):
    model_config = get_configuration_dict(
        json_encoders={datetime: lambda v: v.isoformat()},
    )

    company_listing: UniqueCompanyListing = Field(
        ..., description="The company listing to use for the SWOT analysis"
    )
    use_earnings_call: bool = Field(
        default=False, description="Whether to use earnings calls as a data source"
    )
    use_web_sources: bool = Field(
        default=False, description="Whether to use web sources as a data source"
    )
    earnings_call_start_date: datetime | None = Field(
        default=None, description="The metadata filter to use for the data source"
    )

    def render_session_info(self, state: SessionState = SessionState.RUNNING) -> str:
        data = self.model_dump()
        # Format datetime to yyyy-mm-dd
        data["state"] = state
        date = data.get("earnings_call_start_date")
        if date:
            date = date.strftime("%Y-%m-%d")
            data["earnings_call_start_date"] = date
        return Template(SESSION_INFO_TEMPLATE).render(swot_analysis=data)


class SessionConfig(BaseModel):
    model_config = get_configuration_dict()

    swot_analysis: SwotAnalysisSessionConfig



"""Tests for the SwotAnalysisToolConfig."""

from unique_swot.config import SwotAnalysisToolConfig


def test_config_cache_scope_id_has_default():
    """Test that cache_scope_id has a default empty string value."""
    # Create config without specifying cache_scope_id
    config = SwotAnalysisToolConfig()

    # Verify the default value
    assert config.cache_scope_id == ""
    assert isinstance(config.cache_scope_id, str)


def test_config_cache_scope_id_can_be_set():
    """Test that cache_scope_id can be explicitly set."""
    # Create config with a specific cache_scope_id
    test_scope_id = "test-scope-123"
    config = SwotAnalysisToolConfig(cache_scope_id=test_scope_id)

    # Verify the value is set correctly
    assert config.cache_scope_id == test_scope_id


def test_config_has_required_nested_configs():
    """Test that the config includes all required nested configurations."""
    config = SwotAnalysisToolConfig()

    # Verify nested config attributes exist
    assert hasattr(config, "source_management_config")
    assert hasattr(config, "report_generation_config")
    assert hasattr(config, "report_summarization_config")
    assert hasattr(config, "report_renderer_config")
    assert hasattr(config, "language_model")


def test_config_tool_description_fields_exist():
    """Test that tool description fields are present in the config."""
    config = SwotAnalysisToolConfig()

    # Verify tool description fields
    assert hasattr(config, "tool_description")
    assert hasattr(config, "tool_description_for_system_prompt")
    assert hasattr(config, "tool_format_information_for_system_prompt")
    assert hasattr(config, "tool_description_for_user_prompt")
    assert hasattr(config, "tool_format_information_for_user_prompt")
    assert hasattr(config, "tool_format_reminder_for_user_prompt")

    # Verify they are strings
    assert isinstance(config.tool_description, str)
    assert isinstance(config.tool_description_for_system_prompt, str)



from pydantic import BaseModel, Field
from unique_toolkit._common.pydantic_helpers import get_configuration_dict

from unique_swot.services.generation.agentic.executor import ExecutionMode
from unique_swot.services.generation.agentic.prompts.config import AgenticPromptsConfig


class AgenticGeneratorConfig(BaseModel):
    model_config = get_configuration_dict()

    execution_mode: ExecutionMode = Field(
        default=ExecutionMode.CONCURRENT,
        description="The execution mode to use for the agentic generator.",
    )
    max_concurrent_tasks: int = Field(
        default=10,
        description="The maximum number of concurrent tasks to use for the agentic generator. Only used if execution mode is concurrent.",
    )
    max_tokens_per_extraction_batch: int = Field(
        default=262_144,
        description="The maximum number of tokens to use for the extraction batch. Only used if execution mode is sequential.",
    )
    prompts_config: AgenticPromptsConfig = Field(
        default=AgenticPromptsConfig(),
        description="The prompts config for the agentic generator.",
    )



"""Tests for the SourceSelectionAgent."""

from unittest.mock import AsyncMock, Mock, patch

import pytest
from unique_toolkit.content import Content, ContentChunk

from unique_swot.services.source_management.selection.agent import SourceSelectionAgent
from unique_swot.services.source_management.selection.schema import (
    SourceSelectionResult,
)


def _make_content():
    """Helper to create test Content."""
    return Content(
        id="content_1",
        title="Test Document",
        key="test.pdf",
        chunks=[
            ContentChunk(
                id="content_1",
                chunk_id="chunk_1",
                title="Test Document",
                key="test.pdf",
                text="This document discusses ACME Corp's market position.",
                start_page=1,
                end_page=1,
                order=0,
            )
        ],
    )


@pytest.mark.asyncio
async def test_selection_agent_selects_relevant_source(
    mock_language_model_service, mock_llm, mock_step_notifier
):
    """Test that agent selects relevant sources."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt for {{company_name}}"
    config.max_number_of_selected_chunks = 5

    agent = SourceSelectionAgent(
        llm_service=mock_language_model_service,
        llm=mock_llm,
        source_selection_config=config,
    )

    content = _make_content()

    # Mock the LLM response
    mock_result = SourceSelectionResult(
        should_select=True,
        reason="Relevant to company analysis",
        notification_message="Selected document",
    )

    with patch(
        "unique_swot.services.source_management.selection.agent.generate_structured_output",
        new_callable=AsyncMock,
        return_value=mock_result,
    ):
        result = await agent.select(
            company_name="ACME Corp",
            content=content,
            step_notifier=mock_step_notifier,
        )

        assert result.should_select is True
        assert result.reason == "Relevant to company analysis"


@pytest.mark.asyncio
async def test_selection_agent_rejects_irrelevant_source(
    mock_language_model_service, mock_llm, mock_step_notifier
):
    """Test that agent rejects irrelevant sources."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt"
    config.max_number_of_selected_chunks = 5

    agent = SourceSelectionAgent(
        llm_service=mock_language_model_service,
        llm=mock_llm,
        source_selection_config=config,
    )

    content = _make_content()

    # Mock the LLM response for irrelevant content
    mock_result = SourceSelectionResult(
        should_select=False,
        reason="Not relevant to analysis",
        notification_message="Skipped document",
    )

    with patch(
        "unique_swot.services.source_management.selection.agent.generate_structured_output",
        new_callable=AsyncMock,
        return_value=mock_result,
    ):
        result = await agent.select(
            company_name="ACME Corp",
            content=content,
            step_notifier=mock_step_notifier,
        )

        assert result.should_select is False
        assert result.reason == "Not relevant to analysis"


@pytest.mark.asyncio
async def test_selection_agent_handles_llm_failure(
    mock_language_model_service, mock_llm, mock_step_notifier
):
    """Test that agent handles LLM failures gracefully."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt"
    config.max_number_of_selected_chunks = 5

    agent = SourceSelectionAgent(
        llm_service=mock_language_model_service,
        llm=mock_llm,
        source_selection_config=config,
    )

    content = _make_content()

    # Mock LLM failure (returns None)
    with patch(
        "unique_swot.services.source_management.selection.agent.generate_structured_output",
        new_callable=AsyncMock,
        return_value=None,
    ):
        result = await agent.select(
            company_name="ACME Corp",
            content=content,
            step_notifier=mock_step_notifier,
        )

        # Should default to selecting the source
        assert result.should_select is True
        assert "error" in result.reason.lower() or "occured" in result.reason.lower()


@pytest.mark.asyncio
async def test_selection_agent_sends_notifications(
    mock_language_model_service, mock_llm, mock_step_notifier
):
    """Test that notifications are sent during selection."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt"
    config.max_number_of_selected_chunks = 5

    agent = SourceSelectionAgent(
        llm_service=mock_language_model_service,
        llm=mock_llm,
        source_selection_config=config,
    )

    content = _make_content()

    mock_result = SourceSelectionResult(
        should_select=True,
        reason="Relevant",
        notification_message="Selected",
    )

    with patch(
        "unique_swot.services.source_management.selection.agent.generate_structured_output",
        new_callable=AsyncMock,
        return_value=mock_result,
    ):
        await agent.select(
            company_name="ACME Corp",
            content=content,
            step_notifier=mock_step_notifier,
        )

        # Verify notifications were sent
        assert mock_step_notifier.notify.await_count >= 1


@pytest.mark.asyncio
async def test_selection_agent_limits_chunks(
    mock_language_model_service, mock_llm, mock_step_notifier
):
    """Test that agent respects max_number_of_selected_chunks."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt with {{chunks}}"
    config.max_number_of_selected_chunks = 2

    agent = SourceSelectionAgent(
        llm_service=mock_language_model_service,
        llm=mock_llm,
        source_selection_config=config,
    )

    # Create content with many chunks
    content = Content(
        id="content_1",
        title="Test Document",
        key="test.pdf",
        chunks=[
            ContentChunk(
                id="content_1",
                chunk_id=f"chunk_{i}",
                title="Test Document",
                key="test.pdf",
                text=f"Chunk {i} content",
                start_page=i,
                end_page=i,
                order=i,
            )
            for i in range(10)
        ],
    )

    mock_result = SourceSelectionResult(
        should_select=True,
        reason="Relevant",
        notification_message="Selected",
    )

    with patch(
        "unique_swot.services.source_management.selection.agent.generate_structured_output",
        new_callable=AsyncMock,
        return_value=mock_result,
    ) as mock_generate:
        await agent.select(
            company_name="ACME Corp",
            content=content,
            step_notifier=mock_step_notifier,
        )

        # Verify that only limited chunks were included in the prompt
        # This is implementation-specific, but we can verify the call was made
        mock_generate.assert_awaited_once()



from pathlib import Path
from typing import Annotated

from pydantic import BaseModel, Field
from unique_toolkit._common.pydantic.rjsf_tags import RJSFMetaTag
from unique_toolkit._common.pydantic_helpers import get_configuration_dict

from unique_swot.utils import load_template

PARENT_DIR = Path(__file__).parent


_SYSTEM_PROMPT_TEMPLATE: str = load_template(PARENT_DIR, "system_prompt.j2")
_USER_PROMPT_TEMPLATE: str = load_template(PARENT_DIR, "user_prompt.j2")


class SourceIterationPromptConfig(BaseModel):
    model_config = get_configuration_dict()
    objective: str = Field(
        default="Analyze the provided sources and generate an ordered list from the oldest to the newest.",
        description="The objective of the source iteration.",
    )
    system_prompt: Annotated[
        str,
        RJSFMetaTag.StringWidget.textarea(
            rows=len(_SYSTEM_PROMPT_TEMPLATE.split("\n"))
        ),
    ] = Field(
        default=_SYSTEM_PROMPT_TEMPLATE,
        description="The system prompt for the source selection.",
    )
    user_prompt: Annotated[
        str,
        RJSFMetaTag.StringWidget.textarea(rows=len(_USER_PROMPT_TEMPLATE.split("\n"))),
    ] = Field(
        default=_USER_PROMPT_TEMPLATE,
        description="The user prompt for the source selection.",
    )



"""Tests for the SummarizationAgent."""

from unittest.mock import AsyncMock, Mock

import pytest

from unique_swot.services.generation.models.base import (
    SWOTReportComponents,
    SWOTReportComponentSection,
    SWOTReportSectionEntry,
)
from unique_swot.services.summarization.agent import SummarizationAgent


def _make_report_components():
    """Helper to create test SWOTReportComponents."""
    return SWOTReportComponents(
        strengths=[
            SWOTReportComponentSection(
                h2="Market Position",
                entries=[
                    SWOTReportSectionEntry(
                        preview="Strong brand",
                        content="Strong brand recognition [chunk_a]",
                    )
                ],
            )
        ],
        weaknesses=[
            SWOTReportComponentSection(
                h2="Operational Challenges",
                entries=[
                    SWOTReportSectionEntry(
                        preview="High costs", content="High operational costs [chunk_b]"
                    )
                ],
            )
        ],
        opportunities=[],
        threats=[],
    )


@pytest.mark.asyncio
async def test_summarization_agent_generates_summary(
    mock_llm, mock_language_model_service
):
    """Test that summarization agent generates executive summary."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt for {{company_name}}: {{report}}"

    agent = SummarizationAgent(
        llm=mock_llm,
        llm_service=mock_language_model_service,
        summarization_config=config,
    )

    # Mock the chat service response
    mock_response = Mock()
    mock_response.choices = [Mock()]
    mock_response.choices[0].message.content = "Summary text"
    mock_language_model_service.complete_async = AsyncMock(return_value=mock_response)

    summary = await agent.summarize(
        company_name="ACME Corp", markdown_report="Full report content"
    )

    assert "Summary" in summary
    assert isinstance(summary, str)


@pytest.mark.asyncio
async def test_summarization_agent_handles_llm_failure(
    mock_llm, mock_language_model_service
):
    """Test that agent handles LLM failure gracefully."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt"

    agent = SummarizationAgent(
        llm=mock_llm,
        llm_service=mock_language_model_service,
        summarization_config=config,
    )

    # Mock LLM failure
    mock_language_model_service.complete_async = AsyncMock(
        side_effect=Exception("LLM Error")
    )

    summary = await agent.summarize(
        company_name="ACME Corp", markdown_report="Full report"
    )

    # Should return error message
    assert "error" in summary.lower()


@pytest.mark.asyncio
async def test_summarization_agent_remaps_references(
    mock_llm, mock_language_model_service
):
    """Test that agent remaps source references back to chunks."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt"

    agent = SummarizationAgent(
        llm=mock_llm,
        llm_service=mock_language_model_service,
        summarization_config=config,
    )

    mock_response = Mock()
    mock_response.choices = [Mock()]
    mock_response.choices[0].message.content = "See [source1] and [source2]"
    mock_language_model_service.complete_async = AsyncMock(return_value=mock_response)

    summary = await agent.summarize(company_name="ACME Corp", markdown_report="Report")

    # Summary should be returned
    assert "See [source1] and [source2]" in summary


@pytest.mark.asyncio
async def test_summarization_agent_returns_reference_count(
    mock_llm, mock_language_model_service
):
    """Test that agent returns correct reference count."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt"

    agent = SummarizationAgent(
        llm=mock_llm,
        llm_service=mock_language_model_service,
        summarization_config=config,
    )

    mock_response = Mock()
    mock_response.choices = [Mock()]
    mock_response.choices[0].message.content = "Summary"
    mock_language_model_service.complete_async = AsyncMock(return_value=mock_response)

    summary = await agent.summarize(company_name="ACME Corp", markdown_report="Report")

    assert summary == "Summary"


@pytest.mark.asyncio
async def test_summarization_agent_resets_citation_manager(
    mock_llm, mock_language_model_service
):
    """Test that agent resets citation manager after summarization."""
    config = Mock()
    config.prompt_config.system_prompt = "System prompt"
    config.prompt_config.user_prompt = "User prompt"

    agent = SummarizationAgent(
        llm=mock_llm,
        llm_service=mock_language_model_service,
        summarization_config=config,
    )

    mock_response = Mock()
    mock_response.choices = [Mock()]
    mock_response.choices[0].message.content = "Summary"
    mock_language_model_service.complete_async = AsyncMock(return_value=mock_response)

    summary = await agent.summarize(company_name="ACME Corp", markdown_report="Report")

    # Just verify summary is returned
    assert summary == "Summary"



from pydantic import BaseModel, Field
from unique_toolkit._common.pydantic_helpers import get_configuration_dict

from unique_swot.services.source_management.selection.prompts.config import (
    SourceSelectionPromptConfig,
)


class SourceSelectionConfig(BaseModel):
    model_config = get_configuration_dict()

    max_number_of_selected_chunks: int = Field(
        default=50,
        description="The maximum number of chunks to select to make the decision of selecting the source.",
    )
    prompt_config: SourceSelectionPromptConfig = Field(
        default_factory=SourceSelectionPromptConfig,
        description="The configuration for the source selection prompts.",
    )



from pydantic import BaseModel, Field
from unique_toolkit._common.pydantic_helpers import get_configuration_dict

from unique_swot.services.source_management.collection.config import EarningsCallConfig
from unique_swot.services.source_management.iteration.config import (
    SourceIterationConfig,
)
from unique_swot.services.source_management.selection.config import (
    SourceSelectionConfig,
)


class SourceManagementConfig(BaseModel):
    model_config = get_configuration_dict()

    source_selection_config: SourceSelectionConfig = Field(
        default_factory=SourceSelectionConfig,
        description="The configuration for the source selection.",
    )

    earnings_call_config: EarningsCallConfig = Field(
        default_factory=EarningsCallConfig,
        description="The configuration for the earnings call management.",
    )

    source_iteration_config: SourceIterationConfig = Field(
        default_factory=SourceIterationConfig,
        description="The configuration for the date relevancy.",
    )



from enum import StrEnum
from pathlib import Path
from typing import Annotated

from pydantic import BaseModel, Field
from unique_toolkit._common.docx_generator import DocxGeneratorConfig
from unique_toolkit._common.pydantic.rjsf_tags import RJSFMetaTag
from unique_toolkit._common.pydantic_helpers import get_configuration_dict

from unique_swot.utils import load_template

# Get the directory containing this file
_PROMPTS_DIR = Path(__file__).parent


REPORT_BODY_TEMPLATE: str = load_template(_PROMPTS_DIR, "report_body_template.j2")
REPORT_STRUCTURE_TEMPLATE: str = load_template(
    _PROMPTS_DIR, "report_structure_template.j2"
)


class RendererType(StrEnum):
    DOCX = "docx"
    CHAT = "chat"


class ReportRendererConfig(BaseModel):
    model_config = get_configuration_dict()

    report_structure_template: Annotated[
        str,
        RJSFMetaTag.StringWidget.textarea(
            rows=len(REPORT_STRUCTURE_TEMPLATE.split("\n"))
        ),
    ] = Field(
        default=REPORT_STRUCTURE_TEMPLATE,
        description="Jinja2 template for the report structure.",
    )

    report_body_template: Annotated[
        str,
        RJSFMetaTag.StringWidget.textarea(rows=len(REPORT_BODY_TEMPLATE.split("\n"))),
    ] = Field(
        default=REPORT_BODY_TEMPLATE,
        description="Jinja2 template for the report.",
    )

    renderer_type: RendererType = Field(
        default=RendererType.DOCX,
        description="The type of renderer to use.",
    )

    docx_renderer_config: DocxGeneratorConfig = Field(
        default_factory=DocxGeneratorConfig,
        description="The configuration for the DOCX renderer.",
    )
    ingest_docx_report: bool = Field(
        default=False,
        description="Ingesting the DOCX report into the chat allows the user to ask follow-up questions. It also allows side view when clicking on the reference.",
    )



from unittest.mock import Mock

from unique_toolkit.content import ContentChunk

from unique_swot.services.source_management.registry import (
    ContentChunkRegistry,
    RegistryStore,
)


def _make_chunk(chunk_id="test_chunk"):
    """Helper to create test ContentChunk."""
    return ContentChunk(
        id="content_1",
        chunk_id=chunk_id,
        title="Test Doc",
        key="doc.pdf",
        text="Test content",
        start_page=1,
        end_page=1,
        order=0,
    )


def test_register_generates_unique_ids():
    """Test that register() generates unique IDs for chunks."""
    memory_service = Mock()
    memory_service.get.return_value = None
    registry = ContentChunkRegistry(memory_service=memory_service)

    chunk_a = _make_chunk("chunk_a")
    chunk_b = _make_chunk("chunk_b")

    id_a = registry.register(chunk=chunk_a)
    id_b = registry.register(chunk=chunk_b)

    assert id_a.startswith("chunk_")
    assert id_b.startswith("chunk_")
    assert id_a != id_b


def test_register_and_retrieve():
    """Test that registered chunks can be retrieved by ID."""
    memory_service = Mock()
    memory_service.get.return_value = None
    registry = ContentChunkRegistry(memory_service=memory_service)

    chunk = _make_chunk()
    chunk_id = registry.register(chunk=chunk)

    retrieved = registry.retrieve(chunk_id)
    assert retrieved == chunk


def test_retrieve_nonexistent_returns_none():
    """Test that retrieving a non-existent ID returns None."""
    memory_service = Mock()
    memory_service.get.return_value = None
    registry = ContentChunkRegistry(memory_service=memory_service)

    result = registry.retrieve("nonexistent_id")
    assert result is None


def test_save_persists_store_to_memory_service():
    """Test that save() persists the registry store to memory."""
    memory_service = Mock()
    memory_service.get.return_value = None
    registry = ContentChunkRegistry(memory_service=memory_service)

    chunk = _make_chunk()
    registry.register(chunk=chunk)
    registry.save()

    memory_service.set.assert_called_once()
    stored = memory_service.set.call_args.args[0]
    assert isinstance(stored, RegistryStore)
    assert len(stored.items) == 1


def test_registry_initializes_from_existing_store():
    """Test that registry can initialize from an existing store in memory."""
    existing_chunk = _make_chunk()
    existing_store = RegistryStore(items={"chunk_existing": existing_chunk})

    memory_service = Mock()
    memory_service.get.return_value = existing_store

    registry = ContentChunkRegistry(memory_service=memory_service)

    # Should be able to retrieve the existing chunk
    retrieved = registry.retrieve("chunk_existing")
    assert retrieved == existing_chunk


def test_registry_store_property():
    """Test that the store property returns the internal store."""
    memory_service = Mock()
    memory_service.get.return_value = None
    registry = ContentChunkRegistry(memory_service=memory_service)

    store = registry.store
    assert isinstance(store, RegistryStore)
    assert store.items == {}


def test_multiple_registrations_maintain_order():
    """Test that multiple registrations maintain separate entries."""
    memory_service = Mock()
    memory_service.get.return_value = None
    registry = ContentChunkRegistry(memory_service=memory_service)

    chunks = [_make_chunk(f"chunk_{i}") for i in range(5)]
    ids = [registry.register(chunk=chunk) for chunk in chunks]

    # All IDs should be unique
    assert len(set(ids)) == 5

    # All chunks should be retrievable
    for chunk_id, original_chunk in zip(ids, chunks):
        retrieved = registry.retrieve(chunk_id)
        assert retrieved == original_chunk


def test_id_collision_handling():
    """Test that the registry handles ID collisions by retrying."""
    memory_service = Mock()
    memory_service.get.return_value = None
    registry = ContentChunkRegistry(memory_service=memory_service)

    # Register many chunks to increase chance of testing collision logic
    chunks = [_make_chunk(f"chunk_{i}") for i in range(20)]
    ids = [registry.register(chunk=chunk) for chunk in chunks]

    # All IDs should still be unique
    assert len(set(ids)) == 20



from logging import getLogger

from unique_toolkit import ChatService
from unique_toolkit.content.schemas import ContentReference

from unique_swot.services.notification.log_registry import LogRegistry

_LOGGER = getLogger(__name__)


class StepNotifier:
    def __init__(self, chat_service: ChatService):
        self._chat_service = chat_service
        self._log_registry = LogRegistry()

    async def notify(
        self,
        title: str,
        description: str = "",
        sources: list[ContentReference] = [],
        progress: int | None = None,
        completed: bool = False,
    ):
        await self._log_registry.add(
            chat_service=self._chat_service,
            message_id=self._chat_service.assistant_message_id,
            title=title,
            progress=progress,
            description=description,
            sources=sources,
            completed=completed,
        )

    def get_total_number_of_references(self) -> int:
        return self._log_registry._total_number_of_references



from datetime import datetime, timedelta
from logging import getLogger

from typing_extensions import override
from unique_quartr.service import QuartrService
from unique_toolkit import ShortTermMemoryService
from unique_toolkit._common.docx_generator import DocxGeneratorService
from unique_toolkit._common.experimental.endpoint_requestor import RequestorType
from unique_toolkit.agentic.tools.agent_chunks_hanlder import AgentChunksHandler
from unique_toolkit.agentic.tools.factory import ToolFactory
from unique_toolkit.agentic.tools.schemas import ToolCallResponse
from unique_toolkit.agentic.tools.tool import (
    EvaluationMetricName,
    Tool,
)
from unique_toolkit.language_model import (
    LanguageModelFunction,
    LanguageModelMessage,
    LanguageModelMessageRole,
)
from unique_toolkit.language_model.schemas import LanguageModelToolDescription
from unique_toolkit.services.knowledge_base import KnowledgeBaseService

from unique_swot.config import SwotAnalysisToolConfig
from unique_swot.services.citations import CitationManager
from unique_swot.services.generation.agentic.agent import GenerationAgent
from unique_swot.services.generation.agentic.executor import AgenticPlanExecutor
from unique_swot.services.generation.models.registry import SWOTReportRegistry
from unique_swot.services.memory.base import SwotMemoryService
from unique_swot.services.notification.notifier import StepNotifier
from unique_swot.services.notification.progress import ProgressNotifier
from unique_swot.services.orchestrator.service import SWOTOrchestrator
from unique_swot.services.report import ReportDeliveryService, ReportRendererConfig
from unique_swot.services.schemas import SWOTPlan
from unique_swot.services.session import SessionConfig
from unique_swot.services.source_management.collection.base import (
    CollectionContext,
    SourceCollectionManager,
)
from unique_swot.services.source_management.iteration.agent import (
    SourceIterationAgent,
)
from unique_swot.services.source_management.registry import ContentChunkRegistry
from unique_swot.services.source_management.selection.agent import SourceSelectionAgent
from unique_swot.services.summarization.agent import SummarizationAgent

_LOGGER = getLogger(__name__)


class SwotAnalysisTool(Tool[SwotAnalysisToolConfig]):
    name = "SwotAnalysis"

    def __init__(self, configuration: SwotAnalysisToolConfig, *args, **kwargs):
        super().__init__(configuration, *args, **kwargs)

        self._metadata_filter = self._event.payload.metadata_filter

        self._knowledge_base_service = KnowledgeBaseService.from_event(self._event)

        self._short_term_memory_service = ShortTermMemoryService(
            company_id=self._event.company_id,
            user_id=self._event.user_id,
            chat_id=self._event.payload.chat_id,
            message_id=None,
        )

    def _get_document_template(self, template_content_id: str | None) -> bytes | None:
        if not template_content_id:
            return None
        try:
            file_content = self._knowledge_base_service.download_content_to_bytes(
                content_id=template_content_id
            )
        except Exception as e:
            _LOGGER.warning(
                f"An error occurred while downloading the template {e}. Make sure the template content ID is valid. Falling back to default template."
            )
            return None
        return file_content

    def _try_load_session_config(self):
        try:
            return SessionConfig.model_validate(
                self._event.payload.session_config, by_name=True
            )
        except Exception as e:
            _LOGGER.error(f"Error validating session config: {e}")
            return None

    @override
    def tool_description(self) -> LanguageModelToolDescription:
        return LanguageModelToolDescription(
            name=self.name,
            description=self.config.tool_description,
            parameters=SWOTPlan,
        )

    @override
    def tool_description_for_system_prompt(self) -> str:
        return self.config.tool_description_for_system_prompt

    @override
    def tool_format_information_for_system_prompt(self) -> str:
        return self.config.tool_format_information_for_system_prompt

    @override
    def tool_description_for_user_prompt(self) -> str:
        return self.config.tool_description_for_user_prompt

    @override
    def tool_format_information_for_user_prompt(self) -> str:
        return self.config.tool_format_information_for_user_prompt

    @override
    def tool_format_reminder_for_user_prompt(self) -> str:
        return self.config.tool_format_reminder_for_user_prompt

    @override
    async def run(self, tool_call: LanguageModelFunction) -> ToolCallResponse:
        session_config = self._try_load_session_config()
        if not session_config:
            await self._chat_service.modify_assistant_message_async(
                content="Please make sure to provide the mandatory fields in the SWOT Analysis side bar configuration.",
                set_completed_at=True,
            )
            return ToolCallResponse(
                id=tool_call.id,  # type: ignore
                name=self.name,
                content="The user has not provided a valid configuration for the SWOT. He must provide a valid configuration before running the tool.",
                content_chunks=[],
            )
        company_name = session_config.swot_analysis.company_listing.name
        progress_notifier = self._get_progress_notifier(company_name=company_name)

        try:
            plan = SWOTPlan.model_validate(tool_call.arguments)
            # Ensure the plan is semantically valid for execution, as Pydantic validation may accept plans that
            # are structurally correct but logically invalid (e.g., missing required modify instructions)

            plan.validate_swot_plan()

            # TODO: MOVE THIS TO THE STEP NOTIFIER

            # This service is used to store intermediate results of the SWOT analysis
            memory_service = SwotMemoryService(
                short_term_memory_service=self._short_term_memory_service,
                knowledge_base_service=self._knowledge_base_service,
                cache_scope_id=self.config.cache_scope_id,
            )

            # This service is used to store the content chunks and generate unique IDs for them
            # It is used to track the content chunks and generate citations for the report
            content_chunk_registry = self._get_content_chunk_registry(
                memory_service=memory_service
            )

            # This service is used to collect the sources from the knowledge base
            # It is used to collect the sources from the knowledge base
            source_collector = self._get_source_collector(
                knowledge_base_service=self._knowledge_base_service,
                session_config=session_config,
            )

            # This service is used to define what sources are relevant for the SWOT analysis and what are not
            source_selector = self._get_source_selector()

            # This service is used to define the order in which the sources are processed
            source_iterator = self._get_source_iterator()

            # This service is used to generate the report from the extracted information
            reporting_agent = self._get_reporting_agent(company_name=company_name)

            step_notifier = self._get_step_notifier()

            progress_title = session_config.swot_analysis.render_session_info()

            # initialize progress bar
            await progress_notifier.start(title=progress_title)

            # This service is used to orchestrate the SWOT analysis
            orchestrator = SWOTOrchestrator(
                source_collector=source_collector,
                source_iterator=source_iterator,
                source_selector=source_selector,
                step_notifier=step_notifier,
                reporting_agent=reporting_agent,
                memory_service=memory_service,
                source_registry=content_chunk_registry,
                chat_service=self._chat_service,
                progress_notifier=progress_notifier,
            )

            # Generate markdown report
            result = await orchestrator.run(company_name=company_name, plan=plan)

            if result.is_empty():
                await progress_notifier.finish()
                await self._chat_service.modify_assistant_message_async(
                    content=f"No SWOT analysis results found for the {company_name} with the selected sources. Please make sure to select the correct sources and try again.",
                    set_completed_at=True,
                )
                return ToolCallResponse(
                    id=tool_call.id,  # type: ignore
                    name=self.name,
                    content="No SWOT analysis results found for the company. Please try again with a different company or configuration.",
                    content_chunks=[],
                )

            await progress_notifier.update(progress=90)

            citation_manager = self._get_citation_manager(content_chunk_registry)

            report_handler = self._get_report_delivery_service(
                citation_manager=citation_manager,
                report_renderer_config=self.config.report_renderer_config,
            )

            summarization_agent = self._get_summarization_agent()

            executive_summary = report_handler.render_body(
                result=result,
            )

            summarization_result = await summarization_agent.summarize(
                company_name=company_name, markdown_report=executive_summary
            )

            report_handler.deliver_report(
                executive_summary=summarization_result,
                body=executive_summary,
                session_config=session_config.swot_analysis,
                docx_template_fields={
                    "title": f"{company_name} SWOT Analysis Report",
                    "date": datetime.now().strftime("%Y-%m-%d"),
                },
                ingest_docx=self.config.report_renderer_config.ingest_docx_report,
            )

            await progress_notifier.finish()

            return ToolCallResponse(
                id=tool_call.id,  # type: ignore
                name=self.name,
                content=summarization_result,
                content_chunks=citation_manager.get_referenced_content_chunks(),
            )

        except Exception as e:
            await progress_notifier.finish(failed=True)

            await self._chat_service.modify_assistant_message_async(
                content=f"Error running SWOT Analysis for {company_name}: {e}",
            )
            _LOGGER.exception(f"Error running SWOT plan: {e}")
            return ToolCallResponse(
                id=tool_call.id,  # type: ignore
                name=self.name,
                content=f"Error running SWOT plan: {e}",
                content_chunks=[],
            )

        finally:
            await self._chat_service.modify_assistant_message_async(
                set_completed_at=True,
            )

    @override
    def get_evaluation_checks_based_on_tool_response(
        self, tool_response: ToolCallResponse
    ) -> list[EvaluationMetricName]:
        return []

    @override
    def evaluation_check_list(self) -> list[EvaluationMetricName]:
        return []

    @override
    def get_tool_call_result_for_loop_history(  # type: ignore
        self,
        tool_response: ToolCallResponse,
        agent_chunks_handler: AgentChunksHandler,
    ) -> LanguageModelMessage:
        return LanguageModelMessage(
            role=LanguageModelMessageRole.TOOL,
            content=tool_response.content,
        )

    @override
    def takes_control(self) -> bool:  # type: ignore
        """
        Some tools require to take control of the conversation with the user and do not want the orchestrator to intervene.
        this function indicates whether the tool takes control or not. It yanks the control away from the orchestrator.
        A typical use-case is deep-research.
        """
        return True

    @staticmethod
    def _get_quartr_service(company_id: str) -> QuartrService | None:
        try:
            return QuartrService(
                company_id=company_id,
                requestor_type=RequestorType.REQUESTS,
            )
        except Exception as e:
            _LOGGER.error(f"Error getting Quartr service: {e}")
            return None

    def _get_source_collector(
        self,
        knowledge_base_service: KnowledgeBaseService,
        session_config: SessionConfig,
    ) -> SourceCollectionManager:
        earnings_call_docx_generator_service = DocxGeneratorService(
            config=self.config.source_management_config.earnings_call_config.docx_renderer_config,
            template=self._get_document_template(
                self.config.source_management_config.earnings_call_config.docx_renderer_config.template_content_id
            ),
        )

        collection_context = CollectionContext(
            use_earnings_calls=session_config.swot_analysis.use_earnings_call,
            use_web_sources=session_config.swot_analysis.use_web_sources,
            metadata_filter=self._metadata_filter,
            company=session_config.swot_analysis.company_listing,
            earnings_call_start_date=session_config.swot_analysis.earnings_call_start_date
            or datetime.now() - timedelta(days=365),
            upload_scope_id_earnings_calls=self.config.source_management_config.earnings_call_config.upload_scope_id,
        )

        return SourceCollectionManager(
            context=collection_context,
            knowledge_base_service=knowledge_base_service,
            quartr_service=self._get_quartr_service(self._event.company_id),
            earnings_call_docx_generator_service=earnings_call_docx_generator_service,
        )

    def _get_source_selector(self) -> SourceSelectionAgent:
        return SourceSelectionAgent(
            llm_service=self._language_model_service,
            llm=self.config.language_model,
            source_selection_config=self.config.source_management_config.source_selection_config,
        )

    def _get_source_iterator(self) -> SourceIterationAgent:
        return SourceIterationAgent(
            llm_service=self._language_model_service,
            llm=self.config.language_model,
            source_iteration_config=self.config.source_management_config.source_iteration_config,
        )

    def _get_content_chunk_registry(
        self, memory_service: SwotMemoryService
    ) -> ContentChunkRegistry:
        content_chunk_registry = ContentChunkRegistry(memory_service=memory_service)
        return content_chunk_registry

    def _get_citation_manager(
        self, content_chunk_registry: ContentChunkRegistry
    ) -> CitationManager:
        citation_manager = CitationManager(
            content_chunk_registry=content_chunk_registry
        )
        return citation_manager

    def _get_report_delivery_service(
        self,
        citation_manager: CitationManager,
        report_renderer_config: ReportRendererConfig,
    ) -> ReportDeliveryService:
        report_docx_renderer = DocxGeneratorService(
            config=report_renderer_config.docx_renderer_config,
            template=self._get_document_template(
                report_renderer_config.docx_renderer_config.template_content_id
            ),
        )
        return ReportDeliveryService(
            chat_service=self._chat_service,
            docx_renderer=report_docx_renderer,
            citation_manager=citation_manager,
            renderer_config=report_renderer_config,
        )

    def _get_step_notifier(self) -> StepNotifier:
        return StepNotifier(
            chat_service=self._chat_service,
        )

    def _get_reporting_agent(self, *, company_name: str) -> GenerationAgent:
        executor = AgenticPlanExecutor(
            execution_mode=self.config.report_generation_config.agentic_generator_config.execution_mode,
            max_concurrent_tasks=self.config.report_generation_config.agentic_generator_config.max_concurrent_tasks,
        )
        return GenerationAgent(
            llm=self.config.report_generation_config.language_model,
            llm_service=self._language_model_service,
            registry=SWOTReportRegistry(),
            company_name=company_name,
            executor=executor,
            config=self.config.report_generation_config.agentic_generator_config,
        )

    def _get_summarization_agent(self) -> SummarizationAgent:
        return SummarizationAgent(
            llm=self.config.report_summarization_config.language_model,
            llm_service=self._language_model_service,
            summarization_config=self.config.report_summarization_config,
        )

    def _get_progress_notifier(self, *, company_name: str) -> ProgressNotifier:
        return ProgressNotifier(
            chat_service=self._chat_service,
            company_name=company_name,
        )


ToolFactory.register_tool(tool=SwotAnalysisTool, tool_config=SwotAnalysisToolConfig)



import pytest

from unique_swot.services.generation.context import SWOTComponent
from unique_swot.services.schemas import (
    SWOTOperation,
    SWOTPlan,
    SWOTResult,
    SWOTStepPlan,
)


def _build_plan(
    strengths_op=SWOTOperation.GENERATE,
    weaknesses_op=SWOTOperation.GENERATE,
    opportunities_op=SWOTOperation.GENERATE,
    threats_op=SWOTOperation.GENERATE,
    strengths_instruction=None,
):
    return SWOTPlan(
        objective="Test objective",
        strengths=SWOTStepPlan(
            operation=strengths_op, modify_instruction=strengths_instruction
        ),
        weaknesses=SWOTStepPlan(operation=weaknesses_op, modify_instruction=None),
        opportunities=SWOTStepPlan(operation=opportunities_op, modify_instruction=None),
        threats=SWOTStepPlan(operation=threats_op, modify_instruction=None),
    )


def test_validate_swot_plan_requires_instruction_for_modify():
    plan = _build_plan(strengths_op=SWOTOperation.MODIFY)

    with pytest.raises(ValueError):
        plan.validate_swot_plan()


def test_plan_len_counts_requested_components():
    plan = _build_plan(threats_op=SWOTOperation.NOT_REQUESTED)
    assert len(plan) == 3


def test_swot_result_assignment_and_to_markdown():
    plan = _build_plan()
    result = SWOTResult.init_from_plan(plan=plan)

    result.assign_result(SWOTComponent.STRENGTHS, "Strong brand")
    result.assign_result(SWOTComponent.THREATS, "New entrants")

    assert result.strengths.result == "Strong brand"
    assert result.threats.result == "New entrants"

    markdown = result.to_markdown_report(
        markdown_jinja_template="{{ strengths.result }} | {{ threats.result }}",
        processor=str.upper,
    )
    assert markdown == "STRONG BRAND | NEW ENTRANTS"
